package io.metersphere.keycloak.service;

import com.fasterxml.jackson.core.type.TypeReference;
import io.metersphere.sdk.exception.MSException;
import io.metersphere.sdk.util.JSON;
import io.metersphere.sdk.util.LogUtils;
import io.metersphere.sdk.util.Translator;
import io.metersphere.system.controller.handler.ResultHolder;
import io.metersphere.system.domain.AuthSource;
import io.metersphere.system.domain.User;
import io.metersphere.system.dto.sdk.LoginRequest;
import io.metersphere.system.dto.sdk.SessionUser;
import io.metersphere.system.mapper.BaseUserMapper;
import io.metersphere.system.service.UserLoginService;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.client5.http.impl.io.PoolingHttpClientConnectionManagerBuilder;
import org.apache.hc.client5.http.io.HttpClientConnectionManager;
import org.apache.hc.client5.http.ssl.NoopHostnameVerifier;
import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactory;
import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactoryBuilder;
import org.apache.hc.core5.ssl.SSLContexts;
import org.apache.hc.core5.ssl.TrustStrategy;
import org.apache.shiro.SecurityUtils;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.*;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.net.ssl.SSLContext;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


@Service
@Transactional(rollbackFor = Exception.class)
public class KeycloakService {
    @Resource
    private KeycloakAuthSourceService keycloakAuthSourceService;
    @Resource
    private Environment env;
    @Resource(name = "stringRedisTemplate")
    private StringRedisTemplate stringRedisTemplate;
    @Resource
    private UserLoginService userLoginService;
    @Resource
    private BaseUserMapper baseUserMapper;

    private RestTemplate getRestTemplateIgnoreSSL() throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException {
        TrustStrategy acceptingTrustStrategy = (cert, authType) -> true;
        SSLContext sslContext = SSLContexts.custom().loadTrustMaterial(null, acceptingTrustStrategy).build();
        SSLConnectionSocketFactory sslsf = SSLConnectionSocketFactoryBuilder.create()
                .setSslContext(sslContext)
                .setHostnameVerifier(NoopHostnameVerifier.INSTANCE)
                .build();

//        Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
//                .register("https", sslsf)
//                .register("http", new PlainConnectionSocketFactory())
//                .build();

        HttpClientConnectionManager connectionManager = PoolingHttpClientConnectionManagerBuilder.create()
                .setSSLSocketFactory(sslsf)
                .build();

        CloseableHttpClient httpClient = HttpClients.custom()
                .setConnectionManager(connectionManager)
                .build();

        HttpComponentsClientHttpRequestFactory requestFactory =
                new HttpComponentsClientHttpRequestFactory(httpClient);

        return new RestTemplate(requestFactory);
    }

    public SessionUser exchangeKeycloak(String code, String authId) throws Exception {
        AuthSource authSource = keycloakAuthSourceService.getAuthSource(authId);
        Map<String, String> config = JSON.parseObject(
                new String(authSource.getConfiguration(), StandardCharsets.UTF_8),
                new TypeReference<HashMap<String, String>>() {
                });
        String url = config.get("tokenUrl");

        Map<String, String> resultObj = null;
        try {
            RestTemplate restTemplate = getRestTemplateIgnoreSSL();
            HttpHeaders headers = new HttpHeaders();
            headers.add(HttpHeaders.ACCEPT, MediaType.APPLICATION_JSON_VALUE);
            headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE);

            // 创建表单数据
            MultiValueMap<String, String> form = new LinkedMultiValueMap<>();
            form.add("client_id", config.get("clientId"));
            form.add("client_secret", config.get("secret"));
            form.add("redirect_uri", config.get("redirectUrl"));
            form.add("code", code);
            form.add("grant_type", "authorization_code");

            HttpEntity<MultiValueMap<String, String>> param = new HttpEntity<>(form, headers);
            ResponseEntity<String> response = restTemplate.postForEntity(url, param, String.class);
            String content = response.getBody();
            resultObj = JSON.parseObject(content, new TypeReference<HashMap<String, String>>() {
            });
        } catch (Exception e) {
            LogUtils.error("fail to get access_token", e);
            throw new MSException("fail to get access_token!");
        }

        String accessToken = resultObj.get("access_token");

        if (StringUtils.isBlank(accessToken)) {
            throw new MSException("access_token is empty!");
        }

        return doOauth2Login(authSource, accessToken);
    }

    private SessionUser doOauth2Login(AuthSource authSource, String accessToken) throws Exception {
        Map<String, String> oauth2Config = null;
        Map<String, Object> resultObj = null;
        try {
            oauth2Config = JSON.parseObject(
                    new String(authSource.getConfiguration(), StandardCharsets.UTF_8),
                    new TypeReference<HashMap<String, String>>() {
                    });
            String userInfoUrl = oauth2Config.get("userInfoUrl");
            HttpHeaders headers = new HttpHeaders();
            headers.set("Authorization", "Bearer " + accessToken);
            headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
            RestTemplate restTemplate = getRestTemplateIgnoreSSL();
            HttpEntity<MultiValueMap<String, String>> httpEntity = new HttpEntity<>(headers);
            ResponseEntity<String> response =
                    restTemplate.exchange(userInfoUrl, HttpMethod.POST, httpEntity, String.class);
            resultObj = JSON.parseObject(response.getBody(), new TypeReference<HashMap<String, Object>>() {
            });
            LogUtils.info("user info: " + response.getBody());
        } catch (Exception e) {
            LogUtils.error("fail to get user info", e);
            throw new MSException("fail to get user info!");
        }

        String attrMapping = oauth2Config.get("mapping");
        Map<String, String> mapping = this.getOauth2AttrMapping(attrMapping);

        String userid = (String) resultObj.get(mapping.get("userid"));
        String username = (String) resultObj.get(mapping.get("username"));
        String email = (String) resultObj.get(mapping.get("email"));

        if (StringUtils.isBlank(userid)) {
            throw new MSException("userid is empty!");
        }
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(userid);
        if (m.find()) {
            throw new MSException("userid cannot contain Chinese characters!");
        }

        if (StringUtils.isBlank(username)) {
            username = userid;
        }
        if (!StringUtils.contains(email, "@")) {
            email = null;
        }

        List<User> users = baseUserMapper.selectUserIdByEmailList(Collections.singletonList(email));
        if (CollectionUtils.isEmpty(users)) {
            throw new MSException(String.format("user %s not exists!", email));
        }

        User u = users.get(0);

        LoginRequest loginRequest = new LoginRequest();
        loginRequest.setUsername(u.getId());
        loginRequest.setPassword("nothing");
        loginRequest.setAuthenticate(authSource.getType());
        SecurityUtils.getSubject().getSession().setAttribute("authenticate", authSource.getType());
        ResultHolder login = userLoginService.login(loginRequest);
        return (SessionUser) login.getData();
    }

    private Map<String, String> getOauth2AttrMapping(String mappingStr) {
        Map<String, String> mapping = new HashMap<>();
        try {
            mapping = JSON.parseObject(mappingStr, new TypeReference<HashMap<String, String>>() {
            });
        } catch (Exception e) {
            LogUtils.error("get oauth2 mapping config error!", e);
            throw new MSException(Translator.get("oauth_mapping_config_error"));
        }
        String userid = mapping.get("userid");
        if (StringUtils.isBlank(userid)) {
            throw new MSException(Translator.get("oauth_mapping_value_null") + ": userid");
        }
        String username = mapping.get("username");
        if (StringUtils.isBlank(username)) {
            throw new MSException(Translator.get("oauth_mapping_value_null") + ": username");
        }
        String email = mapping.get("email");
        if (StringUtils.isBlank(email)) {
            throw new MSException(Translator.get("oauth_mapping_value_null") + ": email");
        }
        return mapping;
    }

}
